#!/usr/bin/env python
"""
Program for generating skeleton files for C/C++ projects.
"""

import codecs
from optparse import OptionParser
from mako.runtime import Context
from mako.lookup import TemplateLookup
import pwd
import os
import time
import sys
import string

def setup_context(cstopt, noquestions=False):
    """Prompt user for various customisation informations"""
    userentry = pwd.getpwuid(os.getuid())
    author = userentry[4].split(',')[0]
    login = userentry[0]
    fhandle = open('/etc/mailname')
    if fhandle :
        hostmail = fhandle.readline().strip()
    email = login+'@'+hostmail
    license = 'BDS'
    if cstopt['femdk']:
        owner = 'Cracow University of Technology'
    else:
        owner = author

    organisation = owner
    short_description = "Ask the program author what is it purpose"

    date = time.strftime("%c")
    year = time.strftime("%Y")

    if not noquestions:
        inpt = raw_input("One line program description for usage string?  \n")
        if inpt.strip() != "":
            short_description = inpt
        inpt = raw_input("What is the name of the author? [%s]\n" % author)
        if inpt.strip() != "":
            author = inpt
        inpt = raw_input("What is the email address of the author? [%s]\n" % 
                    email)
        if inpt.strip() != "":
            email = inpt
        inpt = raw_input("What is the license "\
                         "(possible BSD, GPL, LGPL)? [%s]\n" % license)
        if inpt.strip() != "":
            license = inpt
        inpt = raw_input("What is the name of copyright holder? "\
                         "[%s]\n" % owner)

    cstopt['author'] = author
    cstopt['email'] = email
    cstopt['date'] = date
    cstopt['license'] = license
    cstopt['owner'] = owner
    cstopt['year'] = year
    cstopt['organisation'] = organisation
    cstopt['short_description'] = short_description
 
def header_guard(baseneme, prefix=''):
    """Construct header guard from the base name"""
    guard = basename.upper()
    guard=prefix.upper() + '_' + guard + '_HXX'
    return guard

def generate_module(basename, options, custom_options):
    """Generate file for a C++ module: header file and source file.
    """
    if options.femdk:
        prefix='femdk'
    else:
        prefix=''

    custom_options['header_guard'] = header_guard(basename, prefix)
    custom_options['header_name'] = basename+".hxx"

    header_template = mylookup.get_template('mkcxx_header.mako')
    source_template = mylookup.get_template('mkcxx_source.mako')

    header_output  = codecs.open(basename+".hxx", "w", "iso-8859-1")
    source_output  = codecs.open(basename+".cxx", "w", "iso-8859-1")

    ctx = Context(header_output, **custom_options)
    header_template.render_context(ctx)

    ctx = Context(source_output, **custom_options)
    source_template.render_context(ctx)

def generate_program(basename, options, custom_options):
    """Generate file for a C++ program.
    """
    if options.femdk:
        prefix='femdk'
    else:
        prefix=''

    custom_options['header_guard'] = header_guard(basename, prefix)

    program_template = mylookup.get_template('mkcxx_program_source.mako')
    conf_template = mylookup.get_template('mkcxx_program_conf.mako')

    program_output  = codecs.open(basename+".cxx", "w", "iso-8859-1")
    conf_output = codecs.open(basename+'_config.h.in', 'w', "iso-8859-1") 

    ctx = Context(program_output, **custom_options)
    program_template.render_context(ctx)

    ctx = Context(conf_output, **custom_options)
    conf_template.render_context(ctx)

    generate_builder(basename, options, custom_options)

    generate_cmdlparser(basename, options, custom_options)

def generate_builder(basename, options, custom_options):
    """Generate builder file, i.e Makefile, CMakeLists.txt, etc.
    """
    
    buildtype = options.buildtype;

    if buildtype == 'none':
        return
    elif buildtype == 'autotools':
        print "WARNING: autotools builder is not supported yet."\
              " Skipping the request"
        return

    template_name = 'mkcxx_' + options.gentype + '_' + options.buildtype \
                    + '.mako'
    builder_template = mylookup.get_template(template_name)

    outnamemap = {'cmake' : 'CMakeLists.txt',
                  'make' : 'Makefile',
                 }
    outname = outnamemap[options.buildtype]

    builder_output  = codecs.open(outname, "w", "iso-8859-1")

    ctx = Context(builder_output, **custom_options)
    builder_template.render_context(ctx)


def generate_cmdlparser(basename, options, custom_options):
    """Generate command line parser.
    """

    cmdlgener = options.cmdlgener;

    if cmdlgener == 'none':
        return

    template_name = 'mkcxx_program_optdef.mako'
    cmdlparser_template = mylookup.get_template(template_name)

    outname = basename + '_cmdl.def'

    cmdlparser_output  = codecs.open(outname, "w", "iso-8859-1")

    ctx = Context(cmdlparser_output, **custom_options)
    cmdlparser_template.render_context(ctx)

def setup_parser(parser, generators, buildtypes, cmdlgeners):
    """Define options for command line parser.
    """
    cmdlparser.add_option("--template-dir", metavar="DIR", dest="tdir", 
                         help ="Search for template files in DIR directory", 
                         )

    cmdlparser.add_option("--femdk", action="store_true", dest="femdk",
                          help ="Generate skeleton file for FEMDK project"
                         )

    cmdlparser.add_option("--noquestions", action="store_true", 
                          dest="noquestions",
                          help = "Do not ask any questions"
                         )

    gentypes = string.join(map(lambda x : "'%s'"%x, generators.keys()),',')

    cmdlparser.add_option("--type", type="choice", choices=generators.keys(),
                          dest="gentype", 
                          help = """Type of output to generate.
                          Possible types: [ %s ]""" % gentypes 
                         )

    bt = string.join(map(lambda x : "'%s'"%x, buildtypes),',')
    cmdlparser.add_option("--build-type", type="choice", choices=buildtypes,
                          dest="buildtype",
                          help = """Build system to use.
                          Possible build types: [ %s ]""" % bt
                         )
    cmdl = string.join(map(lambda x : "'%s'"%x, cmdlgeners),',')
    cmdlparser.add_option("--cmdl-generator", type="choice", choices=cmdlgeners,
                          dest="cmdlgener",
                          help = """Command line parser generator to use.
                          Possible generators: [ %s ]""" % cmdl 
                         )

    cmdlparser.add_option("--nodir", action="store_true", 
                          dest="nodir",
                          help = "Do not create output directory"
                         )

    cmdlparser.add_option("--prefix", type='string', 
                          dest="prefix",
                          help = """Specify the prefix to be added to output
directory path [prefix/basename]"""
                         )

    cmdlparser.set_defaults(buildtype = 'cmake',
                            gentype = 'module',
                            noquestions = False,
                            femdk = False,
                            tdir = '%TEMPLATEDIR%',
                            nodir = False,
                            prefix='./',
                            cmdlgener = 'none'
                            )

def setup_output_dir(basename, options):
    """Create directory for output files and set is as current.
    """
    path = options.prefix
    if options.nodir is False:
        path = os.path.join(options.prefix, basename) 
        try:
            os.makedirs(path)
        except OSError:
            print "ERROR: The path: '%s' already exists" % path
            print "Either remove or rename it and try again"
            sys.exit(1)
    os.chdir(path)

if __name__ == '__main__':

    usage = """%prog [options] basename\n\n""" \
    """Generate skeleton source and header file for a C++ module or a program."""

    generators = { 'module' : generate_module,
                   'program' : generate_program
                 }
    buildtypes = ['none', 'cmake', 'make', 'autotools']
    
    cmdlgeners = ['none', 'autoopt']

    cmdlparser = OptionParser(usage=usage)

    setup_parser(cmdlparser, generators, buildtypes, cmdlgeners)

    (options, args) = cmdlparser.parse_args()

    if len(args) < 1:
        print "ERROR: missing skeleton base name\n"
        print usage
        sys.exit(1)
    
    basename = args[0]

    dirs = [] # directories for template search
    dirs.append(options.tdir)
    mylookup = TemplateLookup(
            output_encoding = 'iso-8859-1',
            directories=dirs)

    custom_options = {}

    custom_options['femdk'] = options.femdk
    custom_options['basename'] = basename
    custom_options['cmdlgener'] = options.cmdlgener

    setup_output_dir(basename, options)

    setup_context(custom_options, options.noquestions)


    # Generate output
    generators[options.gentype](basename, options, custom_options)
